#! /usr/bin/env python3
import typer

from snyk_tags import __app_name__, __version__

app = typer.Typer()

'''
List all the different project types for Snyk Products
'''

# SAST Command
@app.command(help="List all Snyk Code project types")
def sast():
    snykcmd = typer.style("snyk-tags apply sast", bold=True, fg=typer.colors.MAGENTA)
    typer.echo(f"These are all the Snyk Code project types you can use with {snykcmd}")
    typer.echo("\n sast")

# Container Command
@app.command(help="List all Snyk Container project types")
def container():
    snykcmd = typer.style("snyk-tags apply container", bold=True, fg=typer.colors.MAGENTA)
    typer.echo(f"These are all the Snyk Container project types you can use with {snykcmd}")
    typer.echo("\n dockerfile\n apk\n deb\n rpm\n linux")

# IaC Command
@app.command(help="List all Snyk IaC project types")
def iac():
    snykcmd = typer.style("snyk-tags apply iac", bold=True, fg=typer.colors.MAGENTA)
    typer.echo(f"These are all the Snyk IaC project types you can use with {snykcmd}")
    typer.echo("\n terraformconfig\n terraformplan\n k8sconfig\n helmconfig\n cloudformationconfig\n armconfig")

# SCA Command
@app.command(help="List all Snyk Open Source project types")
def sca():
    snykcmd = typer.style("snyk-tags apply sca", bold=True, fg=typer.colors.MAGENTA)
    typer.echo(f"These are all the Snyk Open Source project types you can use with {snykcmd}")
    typer.echo("\n maven\n npm\n nuget\n gradle\n pip\n yarn\n gomodules\n rubygems\n composer\n sbt\n golangdep\n cocoapods\n poetry\n govendor\n cpp\n yarn-workspace\n hex\n paket\n golang")


#! /usr/bin/env python3

import logging
import httpx
import typer

from snyk_tags import __app_name__, __version__

logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
    )

# Apply tags to a specific project
def apply_tag_to_project(
    client: httpx.Client, org_id: str, project_id: str, tag: str, key: str
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }

    req = client.post(f"org/{org_id}/project/{project_id}/tags", data=tag_data)
    
    if req.status_code == 200:
        logging.info(f"Successfully added {tag} tags to Project ID: {project_id}.")

    if req.status_code == 422:
        logging.warning(f"{tag} tag is already applied for Project ID: {project_id}.")

    if req.status_code == 404:
        logging.error(f"Project not found, likely a READ-ONLY project. Project ID: {project_id}. Error message: {req.json()}.")
    
    return req.status_code, req.json()

#
def apply_tags_to_projects(token: str, org_ids: list, name: str, tag: str, key: str) -> None:
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            for project in projects.get("projects"):
                if project["name"].startswith(name):
                    apply_tag_to_project(
                            client=client, org_id=org_id, project_id=project["id"], tag=tag, key=key
                        )

repoexample = typer.style("'snyk-labs/nodejs-goof'", bold=True, fg=typer.colors.MAGENTA)

@app.command(help=f"Apply a custom tag to a project collection\n\n Use the name you see in the collection as the name: name={repoexample} to tag everything under that repo or CLI import")
def tag(org_id: str = typer.Option(
            ..., # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one or more Organization ID where you want to apply the tag"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  collectionName: str = typer.Option(
            ..., # Default value of comamand
            help=f"Name of the project collection, for example {repoexample}"
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):
    typer.secho(f"\nAdding the tag key {tagKey} and tag value {tagValue} to projects within {collectionName} for easy filtering via the UI", bold=True)
    apply_tags_to_projects(token,[org_id], collectionName, tagValue, tagKey)


#! /usr/bin/env python3

import logging
import httpx
import typer

from snyk_tags import __app_name__, __version__


logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
)

app = typer.Typer()

# Reach to the API and generate tokens
def create_client(token: str) -> httpx.Client:
    return httpx.Client(
        base_url="https://snyk.io/api/v1", headers={"Authorization": f"token {token}"}
    )


def get_org_ids(token: str, group_id: str) -> list:
    """
    Get a list of org_ids based on the given group_id
    :param token:
    :param group_id:
    :return: list of org_ids
    """
    org_ids = []

    with create_client(token=token) as client:
        req = client.get(f"group/{group_id}/orgs")
        if req.status_code == 404:
            logging.error(
            f"Group id: {group_id} is invalid. Error message: {req.json()}."
        )
        orgs = client.get(f"group/{group_id}/orgs").json()
        
        for org in orgs.get("orgs"):
            org_ids.append(org["id"])
    return org_ids

# Apply tags
def apply_tag_to_project(
    client: httpx.Client, org_id: str, project_id: str, tag: str, key: str
) -> tuple:
    tag_data = {
        "key": key,
        "value": tag,
    }
    req = client.post(f"org/{org_id}/project/{project_id}/tags", data=tag_data)

    if req.status_code == 200:
        logging.info(f"Successfully added {tag} tags to Project ID: {project_id}.")

    if req.status_code == 422:
        logging.warning(f"{tag} tag is already applied for Project ID: {project_id}.")

    if req.status_code == 404:
        logging.error(f"Project not found, likely a READ-ONLY project. Project ID: {project_id}. Error message: {req.json()}.")

    return req.status_code, req.json()

def apply_tags_to_projects(token: str, org_ids: list, type: str, tag: str, key: str) -> None:
    with create_client(token=token) as client:
        for org_id in org_ids:
            projects = client.post(f"org/{org_id}/projects").json()
            for project in projects.get("projects"):
                if project["type"] == type:
                    logging.debug(
                        apply_tag_to_project(
                            client=client, org_id=org_id, project_id=project["id"], tag=tag, key=key
                        )
                    )

# SAST Command
sasttypes = typer.style("\n sast", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply Code tag to Snyk Code projects (default: sast")
def sast(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ), token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  sastType: str = typer.Option(
            "sast", # Default value of comamand
            help=f"Type of Snyk Code projects to apply tags to: {sasttypes}"
        )
    ):

    typer.secho(f"\nAdding the Code tag to {sastType} projects in Snyk for easy filtering via the UI", bold=True)

    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, type=sastType, tag='Code', key='Product')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, type=sastType, tag='Code', key='Product')

# IaC Command
iactypes = typer.style("\n terraformconfig\n terraformplan\n k8sconfig\n helmconfig\n cloudformationconfig\n armconfig", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply IaC tag to Snyk IaC projects (default: terraformconfig")
def iac(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ), token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  iacType: str = typer.Option(
            "terraformconfig", # Default value of comamand
            help=f"Type of Snyk IaC projects to apply tags to: {iactypes}"
        )
    ):

    typer.secho(f"\nAdding the IaC tag to {iacType} projects in Snyk for easy filtering via the UI", bold=True)

    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, type=iacType, tag='IaC', key='Product')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, type=iacType, tag='IaC', key='Product')

# SCA Command
scatypes = typer.style("\nmaven\n npm\n nuget\n gradle\n pip\n yarn\n gomodules\n rubygems\n composer\n sbt\n golangdep\n cocoapods\n poetry\n govendor\n cpp\n yarn-workspace\n hex\n paket\n golang", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply Open Source tag to a type Snyk Open Source projects (default: maven)")
def sca(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand,
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  scaType: str = typer.Option(
            "maven", # Default value of comamand
            help=f"Type of Snyk Open Source projects to apply tags to: {scatypes}"
        )
    ):

    typer.secho(f"\nAdding the Open Source tag to {scaType} projects in Snyk for easy filtering via the UI", bold=True)

    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, scaType, tag='Open Source', key='Product')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, scaType, tag='Open Source', key='Product')

# Container Command
containertypes = typer.style("\n dockerfile\n apk\n deb\n rpm\n linux", bold=True, fg=typer.colors.MAGENTA)
@app.command(help="Apply Container tag to a type Snyk Container projects (default: deb)")
def container(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  containerType: str = typer.Option(
            "deb", # Default value of comamand
            help=f"Type of Snyk Container projects to apply tags to: {containertypes}"
        )
    ):

    typer.secho(f"\nAdding the Container tag to {containerType} projects in Snyk for easy filtering via the UI", bold=True)

    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, containerType, tag='Container', key='Product')
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, containerType, tag='Container', key='Product')

# Custom Command
@app.command(help="Apply custom tags to the preferred project type")
def custom(group_id: str = typer.Option(
            ..., # Default value of comamand
            help="Group ID of the Snyk Group you want to apply the tags to",
            envvar=["GROUP_ID"]
        ),  org_id: str = typer.Option(
            "", # Default value of comamand
            envvar=["ORG_ID"],
            help="Specify one Organization ID to only apply the tag to one organization"
        ),  token: str = typer.Option(
            ..., # Default value of comamand
            help="SNYK API token",
            envvar=["SNYK_TOKEN"]
        ),  projectType: str = typer.Option(
            ..., # Default value of comamand
            help="Type of Snyk project to apply tags to: \n dockerfile\n apk\n deb\n rpm\n linux"
        ),  tagKey: str = typer.Option(
            ..., # Default value of comamand
            help="Tag key: identifier of the tag"
        ),  tagValue: str = typer.Option(
            ..., # Default value of comamand
            help="Tag value: value of the tag"
        )
    ):

    typer.secho(f"\nAdding the tag key {tagKey} and tag value {tagValue} to {projectType} projects in Snyk for easy filtering via the UI", bold=True)
    org = []
    if org_id == '' or None:
        org_ids = get_org_ids(token, group_id)
        apply_tags_to_projects(token, org_ids, projectType, tagValue, tagKey)
    else:
        org.append(org_id)
        apply_tags_to_projects(token, org, projectType, tagValue, tagKey)






# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['snyk_tags']

package_data = \
{'': ['*']}

install_requires = \
['colorama>=0.4.5,<0.5.0',
 'httpx>=0.20.0,<0.21.0',
 'shellingham>=1.4.0,<2.0.0',
 'typer>=0.5.0']

entry_points = \
{'console_scripts': ['snyk-tags = snyk_tags.tags:app']}

setup_kwargs = {
    'name': 'snyk-tags',
    'version': '0.3.3',
    'description': 'Tool designed to add tags in bulk to Snyk projects',
    'long_description': "# Snyk Tags Tool\n\nSnyk Tags is a CLI tool with two purposes:\n\n- Help filter Snyk projects by product type by adding product tags across a Snyk Group or Organization - using ```snyk-tags tag```\n- Help filter Snyk projects by applying tags to a collection of projects (for example a git repo like **snyk-labs/nodejs-goof**) - using ```snyk-tags collection```\n\n### snyk-tags tag\n\n```snyk-tags tag``` is a CLI tool that uses the Snyk Project API to assign tags in bulk to Snyk projects based on the product type.\n\n```snyk-tags tag``` will update all projects of the specified product type within a Snyk Group or Organization with the product's tag.\n\nYou can also specify a custom tag for the products.\n\n### snyk-tags collection\n\n```snyk-tags collection``` uses the Snyk Project API to assign tags to all projects within a collection. A collection encompasses one or more projects in Snyk, for example:\n\n- **snyk-labs/nodejs-goof** is a collection from a git import\n- **library/httpd** is a collection from a container import\n- **/snyk-labs/nodejs-goof** is a collection from a CLI import\n\nOnce you run ```snyk-tags```, go into the UI, naviagate to the projects page and find the tags filter options on the left-hand menu. Select the tag you have applied and you will visualize all projects associated.\n\n## Installation and requirements\n\n### Requirements\n\nRequires Python version above 3.6\n\n### Installation\n\nTo install the simplest way is to use pip:\n\n```bash\npip install snyk-tags\n```\n\nAlternatively you can clone the repo and then run the following commands:\n\n```python\npoetry install # To install dependencies\npython -m snyk-tags # To run snyk-tags\n```\n\n## Examples\n\nI want to filter all my Snyk Code projects to the whole Snyk Group:\n\n``` bash\nsnyk-tags tag sast --group-id=abc --token=abc\n```\n\nI want to filter all my ```npm``` Snyk Open Source projects within a specific Snyk Organization:\n\n``` bash\nsnyk-tags tag sca --scatype=npm --org-id=abc --token=abc\n```\n\nI want to filter all projects within my ```snyk-labs/nodejs-goof``` repo as ```project:snyk```\n\n``` bash\nsnyk-tags collection tag --collectionname=snyk-labs/nodejs-goof --org-id=abc --token=abc --tagkey=project --tagvalue=snyk\n```\n",
    'author': 'EricFernandezSnyk',
    'author_email': 'eric.fernandez@snyk.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/EricFernandezSnyk/snyk-tags-tool',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
